@@ -3,9 +3,10 @@ import logging
 from django.core.management import call_command
 from django.core.management.base import BaseCommand
 
+from app_utils.logging import LoggerAddTag
+
 from ... import __title__
 from ...constants import EVE_CATEGORY_ID_BLUEPRINT
-from ...utils import LoggerAddTag
 
 logger = LoggerAddTag(logging.getLogger(__name__), __title__)
 
