@@ -2,22 +2,23 @@ import datetime as dt
 from typing import Tuple
 
 from bravado.exception import HTTPForbidden, HTTPUnauthorized
+
 from django.contrib.auth.models import User
 from django.db import models
 from django.db.models import Case, F, Q, Value, When
 from django.utils.timezone import now
+from esi.models import Token
+from eveuniverse.models import EveEntity, EveSolarSystem, EveType
 
 from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo
 from allianceauth.services.hooks import get_extension_logger
-from esi.models import Token
-from eveuniverse.models import EveEntity, EveSolarSystem, EveType
+from app_utils.logging import LoggerAddTag
 
 from . import __title__
 from .app_settings import BLUEPRINTS_LOCATION_STALE_HOURS
 from .constants import EVE_TYPE_ID_SOLAR_SYSTEM
 from .helpers import fetch_esi_status
 from .providers import esi
-from .utils import LoggerAddTag
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 
@@ -331,39 +332,45 @@ class LocationManager(models.Manager):
 
 
 class RequestQuerySet(models.QuerySet):
-    def requests_fulfillable_by_user(self, user: User) -> models.QuerySet:
-        ownerships = user.character_ownerships.all()
+    def requests_fulfillable_by_user(
+        self, user: User, character_ownerships=None
+    ) -> models.QuerySet:
+        if not character_ownerships:
+            character_ownerships = user.character_ownerships.select_related("character")
         corporation_ids = {
-            character.character.corporation_id for character in ownerships
+            ownership.character.corporation_id for ownership in character_ownerships
         }
-        character_ownership_ids = {character.pk for character in ownerships}
+        character_ownership_pks = {ownership.pk for ownership in character_ownerships}
         request_query = self.select_related(
             "blueprint__owner", "blueprint__owner__corporation"
         ).filter(
             (
                 Q(blueprint__owner__corporation__corporation_id__in=corporation_ids)
-                | Q(blueprint__owner__character__pk__in=character_ownership_ids)
+                | Q(blueprint__owner__character__pk__in=character_ownership_pks)
             )
             & Q(closed_at=None)
-            & Q(status="OP")
+            & Q(status=self.model.STATUS_OPEN)
         )
         return request_query
 
-    def requests_being_fulfilled_by_user(self, user: User) -> models.QuerySet:
-        ownerships = user.character_ownerships.all()
+    def requests_being_fulfilled_by_user(
+        self, user: User, character_ownerships=None
+    ) -> models.QuerySet:
+        if not character_ownerships:
+            character_ownerships = user.character_ownerships.select_related("character")
         corporation_ids = {
-            character.character.corporation_id for character in ownerships
+            ownership.character.corporation_id for ownership in character_ownerships
         }
-        character_ownership_ids = {character.pk for character in ownerships}
+        character_ownership_pks = {ownership.pk for ownership in character_ownerships}
         request_query = self.select_related(
             "blueprint__owner", "blueprint__owner__corporation"
         ).filter(
             (
                 Q(blueprint__owner__corporation__corporation_id__in=corporation_ids)
-                | Q(blueprint__owner__character__pk__in=character_ownership_ids)
+                | Q(blueprint__owner__character__pk__in=character_ownership_pks)
             )
             & Q(closed_at=None)
-            & Q(status="IP")
+            & Q(status=self.model.STATUS_IN_PROGRESS)
             & Q(fulfulling_user=user)
         )
         return request_query
@@ -380,3 +387,13 @@ class RequestManager(models.Manager):
             "blueprint__eve_type",
             "requesting_user__profile__main_character",
         )
+
+    def open_requests_total_count(self, user: User) -> int:
+        """Return total count of open requests for user"""
+        character_ownerships = user.character_ownerships.select_related("character")
+        return (
+            self.all().requests_fulfillable_by_user(user, character_ownerships).count()
+            + self.all()
+            .requests_being_fulfilled_by_user(user, character_ownerships)
+            .count()
+        )
